@@ -10,7 +10,7 @@ static char buf0[512]; /* buffer of zeros */
  * Expects fd's offset to be 0; may also reset fd's offset to 0.
  * Returns 0 on success, and a positive errno otherwise. */
 int
-falloc(int fd, int len)
+rawfalloc(int fd, int len)
 {
     int i, w;
 
