@@ -3,9 +3,18 @@
 #include <sys/socket.h>
 #include "dat.h"
 
+struct Server srv = {
+    Portdef,
+    NULL,
+    NULL,
+    {
+        Filesizedef,
+    },
+};
+
 
 void
-srv(Srv *s)
+srvserve(Server *s)
 {
     int r;
 
@@ -35,26 +44,14 @@ srv(Srv *s)
 
 
 void
-srvschedconn(Srv *s, conn c)
-{
-    if (c->tickpos > -1) {
-        heapremove(&s->conns, c->tickpos);
-    }
-    if (c->tickat) {
-        heapinsert(&s->conns, c);
-    }
-}
-
-
-void
-srvaccept(Srv *s, int ev)
+srvaccept(Server *s, int ev)
 {
     h_accept(s->sock.fd, ev, s);
 }
 
 
 void
-srvtick(Srv *s, int ev)
+srvtick(Server *s, int ev)
 {
     prottick(s);
 }
